//
// Created by robin on 1/8/18.
//

#include <QDebugStateSaver>
#include <QMetaEnum>
#include "dns-base/QDnsLabel.h"
#include "dns-base/QDnsQuery.h"

namespace QDns
{
    namespace Base
    {
        QDnsQuery::QDnsQuery()
                : m_isValid(true)
                  , m_type(QDnsBase::RecordType::A)
                  , m_class(QDnsBase::RecordClass::IN)
        {
        }

        QDnsQuery::QDnsQuery(const QByteArray &packetData, quint32 pos)
                : QDnsQuery()
        {
            QDataStream dataStream(packetData);
            dataStream.skipRawData(pos);
            parse(packetData, dataStream);
        }

        QDnsQuery::QDnsQuery(const QByteArray &packetData, QDataStream &dataStream)
                : QDnsQuery()
        {
            parse(packetData, dataStream);
        }

        void QDnsQuery::parse(const QByteArray &packetData, QDataStream &dataStream)
        {
            m_name = QDnsLabel::parse(packetData, dataStream);
            if (m_name.isNull() || dataStream.device()->bytesAvailable() < 4)
            {
                m_isValid = false;
            }
            else
            {
                dataStream >> m_type;
                dataStream >> m_class;
            }
        }

        QByteArray QDnsQuery::serialize() const
        {
            QByteArray byteArray;
            QDataStream dataStream(&byteArray, QIODevice::WriteOnly);
            serialize(dataStream);
            return byteArray;
        }

        void QDnsQuery::serialize(QDataStream &dataStream) const
        {
            QDnsLabel::serialize(m_name, dataStream);
            dataStream << m_type;
            dataStream << m_class;
        }

        bool QDnsQuery::isValid() const
        {
            return m_isValid;
        }

        const QString &QDnsQuery::getName() const
        {
            return m_name;
        }

        void QDnsQuery::setName(const QString &name)
        {
            m_name = name;
        }

        QDnsBase::RecordType QDnsQuery::getType() const
        {
            return (QDnsBase::RecordType) m_type;
        }

        void QDnsQuery::setType(QDnsBase::RecordType type)
        {
            m_type = type;
        }

        QDnsBase::RecordClass QDnsQuery::getClass() const
        {
            return (QDnsBase::RecordClass) m_class;
        }

        void QDnsQuery::setClass(QDnsBase::RecordClass aClass)
        {
            m_class = aClass;
        }

        QString QDnsQuery::toString() const
        {
            static int recordClassId = QDns::Base::QDnsBase::staticMetaObject.indexOfEnumerator("RecordClass");
            static int recordTypeId = QDns::Base::QDnsBase::staticMetaObject.indexOfEnumerator("RecordType");
            auto recordClassStr = QString(QDns::Base::QDnsBase::staticMetaObject.enumerator(recordClassId).valueToKey(getClass())).toUpper();
            auto recordTypeStr = QString(QDns::Base::QDnsBase::staticMetaObject.enumerator(recordTypeId).valueToKey(getType())).toUpper();
            return getName() + "\t" + recordClassStr + "\t" + recordTypeStr;
        }

        QDebug operator<<(QDebug debug, const QDnsQuery &q)
        {
            QDebugStateSaver saver(debug);
            debug.noquote() << q.toString();
            return debug;
        }
    }
}