//
// Created by robin on 1/14/18.
//

#include <QIODevice>
#include <QDataStream>
#include "dns-base/RecordData/QDnsRecordDataRaw.h"

namespace QDns
{
    namespace Base
    {

        QDnsRecordDataRaw::QDnsRecordDataRaw()
            : QDnsRecordData()
        {
        }

        QDnsRecordDataRaw::QDnsRecordDataRaw(const QByteArray &packetData, quint32 pos)
        {
            QDataStream dataStream(packetData);
            dataStream.skipRawData(pos);
            parse(packetData, dataStream);
        }

        QDnsRecordDataRaw::QDnsRecordDataRaw(const QByteArray &packetData, QDataStream &dataStream)
        {
            parse(packetData, dataStream);
        }

        QByteArray QDnsRecordDataRaw::serialize() const
        {
            QByteArray byteArray;
            QDataStream dataStream(&byteArray, QIODevice::WriteOnly);
            serialize(dataStream);
            return byteArray;
        }

        void QDnsRecordDataRaw::serialize(QDataStream &dataStream) const
        {
            dataStream.writeRawData(m_data.constData(), m_data.length());
        }

        QDnsRecordData *QDnsRecordDataRaw::clone() const
        {
            auto data = new QDnsRecordDataRaw();
            data->m_isValid = m_isValid;
            data->m_data = m_data;
            return data;
        }

        void QDnsRecordDataRaw::parse(const QByteArray &packetData, QDataStream &dataStream)
        {
            while (!dataStream.atEnd())
            {
                char data[128];
                auto read = dataStream.readRawData(data, sizeof(data));
                m_data.append(data, read);
            }
        }

        const QByteArray &QDnsRecordDataRaw::getData() const
        {
            return m_data;
        }

        void QDnsRecordDataRaw::setData(const QByteArray &data)
        {
            m_data = data;
        }

        QString QDnsRecordDataRaw::toString() const
        {
            return m_data.toHex();
        }
    }
}