//
// Created by robin on 1/14/18.
//

#include <gtest/gtest.h>
#include <QByteArray>
#include <QString>
#include <QtNetwork/QHostAddress>
#include <dns-base/RecordData/QDnsRecordDataSOA.h>

struct DnsRDataSOATestParams
{
    bool isValid;
    const char* hexData;
    quint32 pos;

    QString primaryNs;
    QString adminMb;
    quint32 serialNumber;
    quint32 refreshInterval;
    quint32 retryInterval;
    quint32 expirationLimit;
    quint32 minimumTtl;
};

class DnsRDataSOATest : public ::testing::TestWithParam<DnsRDataSOATestParams>
{
};

TEST_P(DnsRDataSOATest, parseAndGetters)
{
    auto params = GetParam();
    auto data = QByteArray::fromHex(params.hexData);
    auto rdata = dynamic_cast<QDns::Base::QDnsRecordDataSOA*>(QDns::Base::QDnsRecordDataSOA(data, params.pos).clone());
    ASSERT_NE(rdata, nullptr);
    ASSERT_EQ(rdata->isValid(), params.isValid);

    if (rdata->isValid())
    {
        ASSERT_EQ(params.primaryNs, rdata->getPrimaryNs());
        ASSERT_EQ(params.adminMb, rdata->getAdminMb());
        ASSERT_EQ(params.serialNumber, rdata->getSerialNumber());
        ASSERT_EQ(params.refreshInterval, rdata->getRefreshInterval());
        ASSERT_EQ(params.retryInterval, rdata->getRetryInterval());
        ASSERT_EQ(params.expirationLimit, rdata->getExpirationLimit());
        ASSERT_EQ(params.minimumTtl, rdata->getMinimumTtl());

        ASSERT_EQ(rdata->serialize().toHex(), QString(params.hexData).remove(" "));
    }
    delete rdata;
}

INSTANTIATE_TEST_CASE_P(DnsRDataSOATestInst,
                        DnsRDataSOATest,
                        ::testing::Values(
                                DnsRDataSOATestParams {false, "", 0},
                                DnsRDataSOATestParams {false, "00", 0},
                                DnsRDataSOATestParams {true, "036e733106676f6f676c6503636f6d0009646e732d61646d696e06676f6f676c6503636f6d000ae6ef6f0000038400000384000007080000003c", 0, "ns1.google.com", "dns-admin.google.com", 182906735, 900, 900, 1800, 60}
                        ));