//
// Created by robin on 2/2/18.
//

#ifndef QDNSAGG_QDNSCLIENT_H
#define QDNSAGG_QDNSCLIENT_H

#include <QObject>
#include <QHostAddress>
#include "dns-base/QDnsPacket.h"

namespace QDns
{
    namespace Client
    {
        class QDnsClient : public QObject
        {
            Q_OBJECT
        public:
            enum ClientState
            {
                Disconnected = 0,
                Connecting = 1,
                Connected = 2,
                Disconnecting = 3
            };

            enum ErrorCode
            {
                Unknown = 0,
                Timeout = 1,
                InvalidData = 2,
                UnknownId = 3,
                NotAnswer = 4,
                SocketError = 5,
                NotConnected = 6
            };

            struct OnStateChangedParams
            {
                quint64 reserved;
                ClientState state;
            };

            struct OnAnswerParams
            {
                quint64 reserved;
                QDns::Base::QDnsPacket query;
                QDns::Base::QDnsPacket answer;
                QHostAddress host;
                quint16 port;
            };

            struct OnErrorParams
            {
                quint64 reserved;
                ErrorCode error;
                QAbstractSocket::SocketError socketError;
                QDns::Base::QDnsPacket query;
                QByteArray data;
                QDns::Base::QDnsPacket answer;
                QHostAddress host;
                quint16 port;
            };

            explicit QDnsClient(const QHostAddress& host, quint16 port = 53, QObject* parent = nullptr);

            const QHostAddress &getHost() const;

            quint16 getPort() const;

            quint32 getQueryTimeout() const;

            void setQueryTimeout(quint32 queryTimeout);

            ClientState getState() const;

        signals:
            void onStateChanged(OnStateChangedParams params);
            void onAnswer(OnAnswerParams params);
            void onError(OnErrorParams params);

        public slots:
            virtual void connectToHost() = 0;
            virtual void query(QDns::Base::QDnsPacket packet);
            virtual void disconnectFromHost() = 0;

        protected:
            struct ProcessDataParams
            {
                QByteArray data;
                QHostAddress senderAddress;
                quint16 senderPort;
            };

            virtual void doQuery(QDns::Base::QDnsPacket packet) = 0;

            void processData(ProcessDataParams params);

            void setState(ClientState state);

            QHostAddress m_host;

            quint16 m_port;

            quint32 m_queryTimeout;

            QMap<quint16, QDns::Base::QDnsPacket> m_pendingQueries;

            ClientState m_state;
        };
    }
}

#endif //QDNSAGG_QDNSCLIENT_H
