//
// Created by robin on 1/24/18.
//

#include "dns-server/QDnsUdpServer.h"

namespace QDns
{
    namespace Server
    {
        QDnsUdpServer::QDnsUdpServer(QObject* parent)
            : QDnsServer(parent)
            , m_socket(new QUdpSocket(this))
        {
            connect(m_socket, &QUdpSocket::readyRead, this, &QDnsUdpServer::onSocketReadyRead);
        }

        bool QDnsUdpServer::start(const QHostAddress &host, quint16 port)
        {
            return m_socket->bind(host, port);
        }

        void QDnsUdpServer::onSocketReadyRead()
        {
            while (m_socket->hasPendingDatagrams())
            {
                QByteArray datagram;
                datagram.resize(m_socket->pendingDatagramSize());
                QHostAddress senderAddress;
                quint16 senderPort;

                m_socket->readDatagram(datagram.data(), datagram.size(), &senderAddress, &senderPort);
                QDns::Base::QDnsPacket packet(datagram);
                if (packet.isValid())
                {
                    emit onRequest(OnRequestParams
                                           {
                                                   packet,
                                                   senderAddress,
                                                   senderPort,
                                                   [this, senderAddress, senderPort](QDns::Server::QDnsServer::SendAnswerParams answerParams)
                                                   {
                                                       sendAnswer(answerParams, senderAddress, senderPort);
                                                   }
                                           });
                }
            }
        }

        void QDnsUdpServer::sendAnswer(const QDns::Server::QDnsServer::SendAnswerParams& answerParams,
                                       const QHostAddress& senderAddress, quint16 senderPort)
        {
            auto answerRaw = answerParams.answer.serialize(); // TODO split packet if too big
            m_socket->writeDatagram(answerRaw, senderAddress, senderPort);
        }
    }
}